###################################################################
###################################################################
##                                                               ##
##   Une proposition de corrigé pour les premières fiches de TP  ##
##                                                               ##
###################################################################
###################################################################



# Il est très important de respecter quelques règles lorsqu'on code:
#
#  1) Une fonction renvoie quelque chose, il faut un RETURN
# 
#  2) Le code doit être abondemment commenté. En particulier, 
#     toute fonction avoir un en-tête qui décrit la nature des
#     paramètres, l'action de la fonction et ce qu'elle renvoit. 



def resEnPara2(R1,R2):
    # R1 et R2 sont deux résistances, en Ohm
    # la fonction renvoie la résistance équivalente
    # au montage en // de R1 et R2
    return(R1*R2/(R1+R2))
        


def resEnPara3(R1,R2,R3):
    # R1, R2 et R3 sont des résistances, en Ohm
    # la fonction renvoie la résistance équivalente
    # au montage en // de R1, R2 et R3
    return(1/(1/R1+1/R2+1/R3))



def resEnPara3bis(R1,R2,R3):
    # R1, R2 et R3 sont des résistances, en Ohm
    # la fonction renvoie la résistance équivalente
    # au montage en // de R1, R2 et R3
    return(resEnPara2(R1,resEnPara2(R2,R3)))
    


def energieC(m,v):
    # la fonction renvoie l'énergie cinétique (J)
    # d'une masse ponctuelle (kg) qui se déplace à la
    # vitesse v (m/s)
    return(0.5*m*v**2)
    
    

def convert(V):
    # renvoie la conversion en m/s d'une vitesse 
    # exprimée en km/h
    return(V/3.6)
    

    
#################################################################################


# pour les racines des polynômes de degré 2, beaucoup de possibilités
# j'en propose 2 : la première, une fonction sans paramètre qui demandera
# à l'utilisateur de saisir les coefficients. On prend la peine de vérifier
# que a est non nul (sinon on aurait une division par 0). On crée systématiquement
# deux racines (éventuellement égales) pour pouvoir avoir un seul return.
    

def racine_poly_deg2():
    # renvoie les racines d'un polynôme de degré 2
    # dont les coefficients seront saisis par l'utilisateur
    a=float(input('saisir le coefficient de degré 2\n'))
    while a==0 :
        print('Sûr que c\'est bien de degré 2?\n')
        a=float(input('saisir un coefficient de degré 2 non nul\n'))
    b=float(input('saisir le coefficient de degré 1\n'))
    c=float(input('saisir le coefficient de degré 0\n'))
    
    # les coefficients du polynôme sont connus, a est non nul
    
    delta=b**2-4*a*c
    
    if delta>=0:
        x1=(-b-delta**0.5)/(2*a)
        x2=(-b+delta**0.5)/(2*a)
            
    else:
        x1=(-b-(-delta)**0.5*1j)/(2*a)
        x2=(-b+(-delta)**0.5*1j)/(2*a)
    
    return(x1,x2)

    

# autre approche, les coefficients sont passés en paramètres de la fonction
# dans l'en-tête on précise que a doit être non nul, si l'utilisateur
# se trompe, tant pis : il n'avait qu'à lire la documentation.


def racine_poly_deg2_bis(a,b,c):
    # a, b et c sont des réels avec a non nul.
    # la fonction renvoie les racines réelles ou complexes conjuguées
    # du polynôme ax^2+bx+c sous forme d'un couple. Si la racine est double, 
    # elle est répétée deux fois. 
    delta=b**2-4*a*c
    
    if delta>=0:
        x1=(-b-delta**0.5)/(2*a)
        x2=(-b+delta**0.5)/(2*a)
            
    else:
        x1=(-b-(-delta)**0.5*1j)/(2*a)
        x2=(-b+(-delta)**0.5*1j)/(2*a)
    
    return(x1,x2)
    


def accesMajeur():
    # la fonction demande son age à l'utilisateur
    # elle renvoie 1 si le nombre entier saisi est >=18, 0 sinon
    age=int(input("Veuillez saisir votre âge svp \n"))
    res=0 
    if age>=18:
        res=1
    return(res)



def fibo(n):
    # n est un entier naturel non nul. La fonction renvoie
    # la liste des n premiers termes de la suite de Fibonacci
    
    # le début de la liste des termes (la réponse si n=2)
    liste=[1,1] 
    
    # si n>2, on construit la liste jusqu'à n termes :
    i=2
    while i<n:
        i=i+1
        liste.append(liste[-1]+liste[-2])
    
    # si n=1, il faut renvoyer [1]
    if n==1: 
        liste=[1]
    
    return(liste)
    


def listeDivis(n):
    # n est un entier naturel non nul. La fonction renvoie la liste de tous
    # les diviseurs de n.
    liste=[]
    i=1
    while i<=n:
        if n%i==0:
            liste.append(i)
        i=i+1
    return(liste)


##############################################################################"
   
    
def sommeEntiers(n):
    # n est un entier naturel.
    # la fonction renvoie la somme des entiers naturels de 0 à n    
    return(n*(n+1)/2)
    


def sommeCubes(n):
    # n est un entier naturel.
    # la fonction renvoie la somme des cubes des entiers naturels de 0 à n    
    somme=0
    for i in range(n+1):
        somme=somme+i**3
    return(somme)
    
    

# Plutôt que de faire avec une boucle, pour la factorielle on programme de 
# façon récursive, càd avec une fonction qui s'appelle elle-même

def factorielle(n):
    # n est un entier naturel non nul.
    # la fonction renvoie la factorielle de n    
    if n==1:
        return(1)
    else:
        return(n*factorielle(n-1))
        
        
    
def sommeDouble(n):
    # n est un entier naturel non nul    
    # la fonction renvoie la somme des ij pour tous les couples
    # (i,j) d'entiers entre 1 et n.
    intervalle=range(1,n+1) 
    somme=0
    
    for i in intervalle:
        for j in intervalle:
            somme=somme+i*j
    
    return(somme)    
    
    

def sommeDouble2(n):
    # n est un entier naturel non nul    
    # la fonction renvoie la somme des ij pour tous les couples
    # (i,j) d'entiers entre 1 et n avec i<=j.
    somme=0
    
    for i in range(1,n+1) :
        for j in range(1,i+1):
            somme=somme+i*j
    
    return(somme)     
    
    

def occurence(x,L):
    # x est un objet, L une liste
    # la fonction renvoie le nombre d'occurences de x dans L
    resultat=0
    for element in L:
        if element==x:
            resultat=resultat+1
    return(resultat)


    
def indice_max(x,L):
    # x est un objet, L une liste
    # la fonction renvoie l'indice maximal de l'objet x dans L
    # si x n'est pas dans L, la fonction renvoie -0.5
    # (les indices  sont des entiers qui peuvent être négatifs,
    # -0.5 n'a pas de sens comme indice)
    indice=-0.5
    longueur=len(L)
    for i in range(longueur):
        if L[i]==x:
            indice=i
    return(indice)
    
    
    
def miroir(L):
    # L est une liste. La fonction renvoie la liste constituée des 
    # éléments de L mais dans l'ordre contraire.
    L_miroir=[]
    for i in L:
        L_miroir=[i]+L_miroir 
        # + est l'opérateur de concaténation sur les listes
    return(L_miroir)



def test_palyndrome(L):
    # L est une liste. 
    # La fonction revoie True si L est un palyndrome, False sinon
    return(L==miroir(L))
    
    
    
def premier(n):
    # n est un entier naturel non nul
    # la fonction renvoie le nombre de nombres premiers qui sont
    # inférieurs ou égaux à n
    resultat=0
    k=2 #plus petit nombre premier
    
    while k<=n:
        if len(listeDivis(k))==2:   # k est premier ssi il a exactement 2 diviseurs
            resultat=resultat+1
        k=k+1
    
    return(resultat)



def meanValue(L):
    # L is a non empty list of numbers
    # the function returns the mean value of L's elements
    
    sumL=0
    for element in L:
        sumL=sumL+element
    #sumL is the sum of L's elements
    
    return(sumL/len(L))
    
    
    
def stdDev(L):
    # L is a non empty list of numbers
    # the function returns the standard deviation of L's elements
    L_mean=meanValue(L)
    
    newList=[]
    for element in L:
        newList.append((element-L_mean)**2)
    
    # the standard deviation of L is the square root of 
    # newList's mean value :
    return(meanValue(newList)**.5)